@@ -1,5 +1,5 @@
 ﻿/*
- * SPDX-FileCopyrightText: 2014-2022 Megan Conkle <megan.conkle@kdemail.net>
+ * SPDX-FileCopyrightText: 2014-2023 Megan Conkle <megan.conkle@kdemail.net>
  * SPDX-FileCopyrightText: 2009-2014 Graeme Gott <graeme@gottcode.org>
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
@@ -29,28 +29,28 @@
 #include <QStatusBar>
 #include <QTextDocumentFragment>
 
-#include <KCoreAddons/KAboutData>
-#include <KXmlGui/KAboutApplicationDialog>
-#include <KXmlGui/KHelpMenu>
+#include <KAboutApplicationDialog>
+#include <KAboutData>
+#include <KHelpMenu>
 
-#include "3rdparty/QtAwesome/QtAwesome.h"
+#include <3rdparty/QtAwesome/QtAwesome.h>
 
-#include "library.h"
-#include "exporter.h"
-#include "exporterfactory.h"
-#include "findreplace.h"
-#include "localedialog.h"
-#include "mainwindow.h"
-#include "messageboxhelper.h"
-#include "preferencesdialog.h"
-#include "previewoptionsdialog.h"
-#include "sandboxedwebpage.h"
-#include "simplefontdialog.h"
-#include "stylesheetbuilder.h"
-#include "themeselectiondialog.h"
+#include "export/exporter.h"
+#include "export/exporterfactory.h"
+#include "settings/localedialog.h"
+#include "settings/preferencesdialog.h"
+#include "settings/previewoptionsdialog.h"
+#include "settings/simplefontdialog.h"
+#include "theme/stylesheetbuilder.h"
+#include "theme/themeselectiondialog.h"
 #include "spelling/spellcheckdecorator.h"
 #include "spelling/spellcheckdialog.h"
 
+#include "findreplace.h"
+#include "library.h"
+#include "mainwindow.h"
+#include "messageboxhelper.h"
+
 namespace ghostwriter
 {
 enum SidebarTabIndex {
@@ -81,7 +81,8 @@ MainWindow::MainWindow(const QString &filePath, QWidget *parent)
     QString themeName = appSettings->themeName();
 
     QString err;
-    theme = ThemeRepository::instance()->loadTheme(themeName, err);
+    ThemeRepository themeRepo(appSettings->themeDirectoryPath());
+    theme = themeRepo.loadTheme(themeName, err);
 
     MarkdownDocument *document = new MarkdownDocument();
 
@@ -119,6 +120,7 @@ MainWindow::MainWindow(const QString &filePath, QWidget *parent)
     documentManager->setAutoSaveEnabled(appSettings->autoSaveEnabled());
     documentManager->setFileBackupEnabled(appSettings->backupFileEnabled());
     documentManager->setDraftLocation(appSettings->draftLocation());
+    documentManager->setBackupLocation(appSettings->backupLocation());
     documentManager->setFileHistoryEnabled(appSettings->fileHistoryEnabled());
     setWindowTitle(documentManager->document()->displayName() + "[*] - " + qAppName());
     connect(documentManager, SIGNAL(documentDisplayNameChanged(QString)), this, SLOT(changeDocumentDisplayName(QString)));
@@ -228,8 +230,10 @@ MainWindow::MainWindow(const QString &filePath, QWidget *parent)
 
     connect(appSettings, SIGNAL(autoSaveChanged(bool)), documentManager, SLOT(setAutoSaveEnabled(bool)));
     connect(appSettings, SIGNAL(backupFileChanged(bool)), documentManager, SLOT(setFileBackupEnabled(bool)));
+    connect(appSettings, SIGNAL(backupLocationChanged(QString)), documentManager, SLOT(setBackupLocation(QString)));
     connect(appSettings, SIGNAL(tabWidthChanged(int)), editor, SLOT(setTabulationWidth(int)));
     connect(appSettings, SIGNAL(insertSpacesForTabsChanged(bool)), editor, SLOT(setInsertSpacesForTabs(bool)));
+    connect(appSettings, &AppSettings::showUnbreakableSpaceEnabledChanged, editor, &MarkdownEditor::setShowUnbreakableSpaces);
     connect(appSettings, SIGNAL(useUnderlineForEmphasisChanged(bool)), editor, SLOT(setUseUnderlineForEmphasis(bool)));
     connect(appSettings, SIGNAL(italicizeBlockquotesChanged(bool)), editor, SLOT(setItalicizeBlockquotes(bool)));
     connect(appSettings, SIGNAL(largeHeadingSizesChanged(bool)), editor, SLOT(setEnableLargeHeadingSizes(bool)));
@@ -246,6 +250,8 @@ MainWindow::MainWindow(const QString &filePath, QWidget *parent)
     connect(appSettings, SIGNAL(previewTextFontChanged(QFont)), this, SLOT(applyTheme()));
     connect(appSettings, SIGNAL(previewCodeFontChanged(QFont)), this, SLOT(applyTheme()));
 
+    editor->setShowUnbreakableSpaces(appSettings->showUnbreakableSpaceEnabled());
+
     if (this->isFullScreen() && appSettings->hideMenuBarInFullScreenEnabled()) {
         this->menuBar()->hide();
     }
@@ -279,7 +285,21 @@ MainWindow::MainWindow(const QString &filePath, QWidget *parent)
         this
     );
 
-    connect(editor, SIGNAL(textChanged()), htmlPreview, SLOT(updatePreview()));
+    connect(editor,
+        &MarkdownEditor::typingPausedScaled,
+        htmlPreview,
+        &HtmlPreview::updatePreview);
+
+    connect(documentManager,
+        &DocumentManager::documentLoaded,
+        htmlPreview,
+        &HtmlPreview::updatePreview);
+
+    connect(documentManager,
+        &DocumentManager::documentClosed,
+        htmlPreview,
+        &HtmlPreview::updatePreview);
+
     connect(outlineWidget, SIGNAL(headingNumberNavigated(int)), htmlPreview, SLOT(navigateToHeading(int)));
     connect(appSettings, SIGNAL(currentHtmlExporterChanged(Exporter *)), htmlPreview, SLOT(setHtmlExporter(Exporter *)));
 
@@ -356,8 +376,6 @@ MainWindow::MainWindow(const QString &filePath, QWidget *parent)
     if (!fileToOpen.isNull() && !fileToOpen.isEmpty()) {
         documentManager->open(fileToOpen);
     }
-
-    spelling->startLiveSpellCheck();
 }
 
 MainWindow::~MainWindow()
@@ -943,6 +961,7 @@ void MainWindow::buildMenuBar()
     formatMenu->addAction(createWidgetAction(tr("&Bold"), editor, SLOT(bold()), QKeySequence::Bold));
     formatMenu->addAction(createWidgetAction(tr("&Italic"), editor, SLOT(italic()), QKeySequence::Italic));
     formatMenu->addAction(createWidgetAction(tr("Stri&kethrough"), editor, SLOT(strikethrough()), QKeySequence("Ctrl+K")));
+    formatMenu->addAction(createWidgetAction(tr("&Code Fences"), editor, SLOT(insertCodeFences()), QKeySequence("Ctrl+G")));
     formatMenu->addAction(createWidgetAction(tr("&HTML Comment"), editor, SLOT(insertComment()), QKeySequence("Ctrl+/")));
     formatMenu->addSeparator();
 
@@ -1403,6 +1422,7 @@ void MainWindow::applyTheme()
 
     StyleSheetBuilder styler(colorScheme,
         (InterfaceStyleRounded == appSettings->interfaceStyle()),
+        appSettings->editorFont(),
         appSettings->previewTextFont(),
         appSettings->previewCodeFont());
 
